@@ -1,9 +1,9 @@
 #ifndef STAN_MATH_PRIM_ARR_FUNCTOR_integrate_1d_HPP
 #define STAN_MATH_PRIM_ARR_FUNCTOR_integrate_1d_HPP
 
+#include <stan/math/prim/meta.hpp>
 #include <stan/math/prim/scal/err/check_less_or_equal.hpp>
 #include <stan/math/prim/scal/err/domain_error.hpp>
-#include <stan/math/prim/scal/meta/scalar_seq_view.hpp>
 #include <boost/math/quadrature/exp_sinh.hpp>
 #include <boost/math/quadrature/sinh_sinh.hpp>
 #include <boost/math/quadrature/tanh_sinh.hpp>
@@ -202,8 +202,9 @@ inline double integrate_1d(
   check_less_or_equal(function, "lower limit", a, b);
 
   if (a == b) {
-    if (std::isinf(a))
+    if (std::isinf(a)) {
       domain_error(function, "Integration endpoints are both", a, "", "");
+    }
     return 0.0;
   } else {
     return integrate(
