@@ -24,14 +24,18 @@ inline Eigen::Matrix<T, Eigen::Dynamic, Eigen::Dynamic> inverse_spd(
   check_symmetric("inverse_spd", "m", m);
   Matrix<T, Dynamic, Dynamic> mmt = T(0.5) * (m + m.transpose());
   LDLT<Matrix<T, Dynamic, Dynamic> > ldlt(mmt);
-  if (ldlt.info() != Eigen::Success)
+  if (ldlt.info() != Eigen::Success) {
     domain_error("invese_spd", "LDLT factor failed", "", "");
-  if (!ldlt.isPositive())
+  }
+  if (!ldlt.isPositive()) {
     domain_error("invese_spd", "matrix not positive definite", "", "");
+  }
   Matrix<T, Dynamic, 1> diag_ldlt = ldlt.vectorD();
-  for (int i = 0; i < diag_ldlt.size(); ++i)
-    if (diag_ldlt(i) <= 0)
+  for (int i = 0; i < diag_ldlt.size(); ++i) {
+    if (diag_ldlt(i) <= 0) {
       domain_error("invese_spd", "matrix not positive definite", "", "");
+    }
+  }
   return ldlt.solve(Eigen::Matrix<T, Eigen::Dynamic, Eigen::Dynamic>::Identity(
       m.rows(), m.cols()));
 }
