@@ -1,9 +1,8 @@
 #ifndef STAN_MATH_PRIM_MAT_FUN_SORT_INDICES_HPP
 #define STAN_MATH_PRIM_MAT_FUN_SORT_INDICES_HPP
 
+#include <stan/math/prim/meta.hpp>
 #include <stan/math/prim/mat/fun/Eigen.hpp>
-#include <stan/math/prim/mat/meta/index_type.hpp>
-#include <stan/math/prim/arr/meta/index_type.hpp>
 #include <algorithm>
 #include <vector>
 
@@ -40,10 +39,11 @@ class index_comparator {
    * @param j Index of second value for comparison
    */
   bool operator()(int i, int j) const {
-    if (ascending)
+    if (ascending) {
       return xs_[i - 1] < xs_[j - 1];
-    else
+    } else {
       return xs_[i - 1] > xs_[j - 1];
+    }
   }
 };
 
@@ -61,12 +61,13 @@ class index_comparator {
  */
 template <bool ascending, typename C>
 std::vector<int> sort_indices(const C& xs) {
-  typedef typename index_type<C>::type idx_t;
+  using idx_t = typename index_type<C>::type;
   idx_t size = xs.size();
   std::vector<int> idxs;
   idxs.resize(size);
-  for (idx_t i = 0; i < size; ++i)
+  for (idx_t i = 0; i < size; ++i) {
     idxs[i] = i + 1;
+  }
   internal::index_comparator<ascending, C> comparator(xs);
   std::sort(idxs.begin(), idxs.end(), comparator);
   return idxs;
