@@ -1,8 +1,10 @@
 #ifndef STAN_MATH_PRIM_SCAL_FUN_LOG1M_EXP_HPP
 #define STAN_MATH_PRIM_SCAL_FUN_LOG1M_EXP_HPP
 
+#include <stan/math/prim/meta.hpp>
 #include <stan/math/prim/scal/fun/expm1.hpp>
 #include <stan/math/prim/scal/fun/log1m.hpp>
+#include <stan/math/prim/scal/fun/constants.hpp>
 #include <cmath>
 #include <limits>
 
@@ -44,12 +46,13 @@ namespace math {
 inline double log1m_exp(double a) {
   using std::exp;
   using std::log;
-  if (a >= 0)
-    return std::numeric_limits<double>::quiet_NaN();
-  else if (a > -0.693147)
+  if (a >= 0) {
+    return NOT_A_NUMBER;
+  } else if (a > -0.693147) {
     return log(-expm1(a));  // 0.693147 ~= log(2)
-  else
+  } else {
     return log1m(exp(a));
+  }
 }
 
 }  // namespace math
