@@ -1,7 +1,7 @@
 #ifndef STAN_MATH_PRIM_SCAL_PROB_VON_MISES_LOG_HPP
 #define STAN_MATH_PRIM_SCAL_PROB_VON_MISES_LOG_HPP
 
-#include <stan/math/prim/scal/meta/return_type.hpp>
+#include <stan/math/prim/meta.hpp>
 #include <stan/math/prim/scal/prob/von_mises_lpdf.hpp>
 
 namespace stan {
@@ -11,8 +11,8 @@ namespace math {
  * @deprecated use <code>von_mises_lpdf</code>
  */
 template <bool propto, typename T_y, typename T_loc, typename T_scale>
-typename return_type<T_y, T_loc, T_scale>::type von_mises_log(
-    T_y const& y, T_loc const& mu, T_scale const& kappa) {
+return_type_t<T_y, T_loc, T_scale> von_mises_log(T_y const& y, T_loc const& mu,
+                                                 T_scale const& kappa) {
   return von_mises_lpdf<propto, T_y, T_loc, T_scale>(y, mu, kappa);
 }
 
@@ -20,8 +20,9 @@ typename return_type<T_y, T_loc, T_scale>::type von_mises_log(
  * @deprecated use <code>von_mises_lpdf</code>
  */
 template <typename T_y, typename T_loc, typename T_scale>
-inline typename return_type<T_y, T_loc, T_scale>::type von_mises_log(
-    T_y const& y, T_loc const& mu, T_scale const& kappa) {
+inline return_type_t<T_y, T_loc, T_scale> von_mises_log(T_y const& y,
+                                                        T_loc const& mu,
+                                                        T_scale const& kappa) {
   return von_mises_lpdf<T_y, T_loc, T_scale>(y, mu, kappa);
 }
 
